SystemOrganization addCategory: #'Coral-Utilities'!

Object subclass: #CoralInstaller
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Coral-Utilities'!

!CoralInstaller commentStamp: 'DamienPollet 7/28/2011 16:04' prior: 0!
A CoralInstaller is responsible to install accessory files to integrate Coral with the shell and environment of the host system.
!

!CoralInstaller class methodsFor: 'script generation' stamp: 'DamienPollet 4/5/2012 17:17'!
computeScriptNameFor: selector
	^selector asString, '.coral'! !

!CoralInstaller class methodsFor: 'file creation' stamp: 'DamienPollet 4/5/2012 16:57'!
createFileNamed: aFileName withContents: aString
	| file |
	file := FileSystem disk referenceTo: aFileName.
	file writeStreamDo: [ :io |
		io truncate; nextPutAll: aString withUnixLineEndings ].
	^ file! !

!CoralInstaller class methodsFor: 'file creation' stamp: 'LucFabresse 1/3/2012 17:20'!
createWrapperNamed: aString withContents: contentString
	| script |
	script := self createFileNamed: aString withContents: contentString.
	self giveExecutionRightToFileNamed:  script pathString
	! !

!CoralInstaller class methodsFor: 'script generation' stamp: 'LucFabresse 1/4/2012 09:39'!
generateAllExamples
	"self generateAllExamples"
	(CoralScriptExamples class allMethodsInCategory: #'script examples') do: [ :selector | 
		|filename file|
		filename := self computeScriptNameFor: selector.
		file := self generateScript: selector to: filename.
		self giveExecutionRightToFileNamed: file pathString
	]

! !

!CoralInstaller class methodsFor: 'script generation' stamp: 'DamienPollet 10/24/2011 18:58'!
generateCoralScript
	"self generateCoralScript"

	self createWrapperNamed: 'coral.sh' withContents:
'#!! /bin/bash
vm=''', Smalltalk vm vmFileName, '''
image=''', (Smalltalk vm imagePath), '''

function usage() {
    echo "Usage: `basename $0` [-di] [script-file.coral]"
    echo "       `basename $0` [-h]"
    echo
    echo "  -d  debug mode: run headful and do not automatically quit"
    echo "  -i  interactive mode: handle input on a character-wise basis instead of line-wise"
    echo "  -h  display this help message"
}

case `uname` in
    Darwin) mktmpdir="mktemp -d -t coral" ;;
    Linux) mktmpdir="mktemp --directory --tmpdir coral-XXXXXXXXXX" ;;
esac

imageArgs=""
vmArgs=""
debug="no"

while getopts ":idh" opt; do
    case $opt in
        i)  # interactive mode, configure the terminal for charwise input
            restore_tty="stty `stty -g`"
            stty -icanon -echo
            ;;
        d)  # debug, headful mode
            debug="yes"
            ;;
        h)
            usage
            exit 0
            ;;
        \?)
            echo "Invalid option: -$OPTARG" >&2
            usage
            exit 1
            ;;
    esac
done
shift $(($OPTIND - 1))

if [[ "$debug" == "yes" ]]; then
    imageArgs+="-debug"
else
    vmArgs+="-headless"
fi
unset debug

export LINES=`tput lines`
export COLUMNS=`tput cols`

# we need to filter stderr but without redirecting it to stdout
# so, setup a named pipe and make sure to clean it afterwards
tmp=`$mktmpdir`
mkfifo "$tmp"/stderr
tail -n +2 "$tmp"/stderr >&2 &

function cleanup() {
    rm -r "$tmp"
    eval $restore_tty
    echo "ok, bye..."
}
trap "cleanup ; trap 2 ; kill -2 $$" 1 2 3 13 15

# finally...
if [[ $# > 0 ]]; then
   script="$1"
   shift
   "$vm" $vmArgs "$image" "`pwd`/$script" $imageArgs --coral "$@" 2>"$tmp"/stderr
else
   "$vm" $vmArgs "$image" "" $imageArgs 2>"$tmp"/stderr
fi

cleanup
'! !

!CoralInstaller class methodsFor: 'script generation' stamp: 'LucFabresse 1/4/2012 09:38'!
generateScript: aSelector to: filename
	^self createFileNamed: filename withContents: '#!!/usr/bin/env coral', String lf, (CoralScriptExamples perform: aSelector)! !

!CoralInstaller class methodsFor: 'file modification' stamp: 'LucFabresse 1/3/2012 17:20'!
giveExecutionRightToFileNamed: fileName
	OSProcess command: 'chmod u+x "' , fileName , '"'
	! !

Object subclass: #CoralScriptExamples
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Coral-Utilities'!

!CoralScriptExamples commentStamp: 'DamienPollet 7/28/2011 16:05' prior: 0!
I'm a library of example Coral scripts.

I could probably be merged into CoralInstaller.!

!CoralScriptExamples class methodsFor: 'script examples' stamp: 'OlivierAuverlot 11/24/2010 14:17'!
scriptCarpet
		"generates a carpet with random colors"

^'[
	Console resetAllAttributes.
	Console eraseScreen.

	(1 to: 80) do: [ :x | 
		(1 to: 20) do: [ :y |
			Console displayAttributes: { ((10 atRandom) + 29) . 7 . ((10 atRandom) + 39) }.
			Console plot: x @ y.
		]
	].
	
	Console resetAllAttributes.

]'! !

!CoralScriptExamples class methodsFor: 'script examples' stamp: 'DamienPollet 4/5/2012 17:18'!
scriptCreationOfFolderAndFiles
	"CoralInstaller generateScript: #scriptCreationOfFolderAndFiles"

	^ '"This script takes one parameter: a path. It will create a folder named CoralGenerated at the path.
If this folder exists, it is deleted first. In CoralGenerated, a few folders and a file are created."

[
	| fs folder |

	fs := FileSystem disk.
	folder := CoralScriptWriting getAllArgumentsWithoutMinusButScriptName at:1 ifAbsent: [ String empty ].
	
	folder isEmpty 
		ifTrue: [ Console<<(''Use: '', CoralScriptWriting getScriptName, '' path'') ]
		ifFalse: [ 
			| ref |

			ref := fs referenceTo: folder.
			ref exists
				ifFalse: [ Console printError: ''The path should exist'' ]
				ifTrue: [
					| stream baseRef |
					baseRef := (ref / ''CoralGenerated'').
					baseRef exists ifTrue: [ baseRef deleteAll ].
					baseRef createDirectory.
					((ref / ''CoralGenerated'') / ''a'') createDirectory.
					((ref / ''CoralGenerated'') / ''b'') createDirectory.
					(((ref / ''CoralGenerated'') / ''b'') / ''c'') createDirectory.
					stream := ((((ref / ''CoralGenerated'') / ''b'') / ''c'') / ''coral.txt'') writeStream.
					stream nextPutAll: ''FileGenerated with Coral''.
					stream close.
					Console<<(CoralScriptWriting getScriptName, '' done.'')
				]
		]
]	
'! !

!CoralScriptExamples class methodsFor: 'script examples' stamp: 'AdrienBarreau 5/10/2011 17:55'!
scriptFewThingsWithOSProcess
	^
'"This script does nothing useful ... but it shows a couple of things you can use while scripting"

[
	| process |
	
	process := OSProcess thisOSProcess.
	
	Console << ''Let''''s take a look at many points of OSProcess!!'' << String lf.
	Console << ''------------------------------------------------------------'' << String lf << String lf.
	
	Console << ''Environment variables:'' << String lf.
	Console << ''------'' << String lf.
	process environment associationsDo: [ :each | Console << each key << '' = '' << each value << String lf ].
	
	Console << String lf << ''You can run shell commands. Here is "ls -la *.cst":'' << String lf.
	Console << ''------'' << String lf.
	process waitForCommand: ''ls -la *.cst''.
	
	Console << String lf << ''You can change the working directory.'' << String lf.
	Console << ''------'' << String lf..
	Console << ''pwd: ''.
	process waitForCommand: ''pwd''.
	Console << ''Changing working directory for /'' << String lf.
	process chDir: ''/''.
	Console << ''pwd: ''.
	process waitForCommand: ''pwd''.
	Console << ''ls:'' << String lf.
	process waitForCommand: ''ls''.

	Console << String lf << CoralScriptWriting getScriptName << '' done.''
]
'! !

!CoralScriptExamples class methodsFor: 'script examples' stamp: 'DamienPollet 4/5/2012 17:19'!
scriptForScriptTestCoralScriptWriting
	"CoralInstaller generateScript: #scriptForScriptTestCoralScriptWriting"

	^ '[
	| fs os |
	
	fs := Filesystem disk.
	os := ThisOSProcess thisOSProcess.
	
	os chDir: (fs referenceTo: ''.'') asAbsolute asString.
	CoralInstaller generateScript: #scriptTestCoralScriptWriting.
	
	os waitForCommand: ''./coral.sh scriptTestCoralScriptWriting.cst arg1 arg2 -minus1 arg3 -minus2 -minus3 arg4 arg5''.
	
	(fs referenceTo: ''./scriptTestCoralScriptWriting.cst'') delete.
	
	Console << (CoralScriptWriting getScriptName, '' done.'')
]	
'! !

!CoralScriptExamples class methodsFor: 'script examples' stamp: 'OlivierAuverlot 11/24/2010 10:41'!
scriptHappyFace

  ^'[
	Console resetAllAttributes.
	Console displayAttributes: #(31 7 42).
	Console eraseScreen.

	Console locate: 10@5; print: ''  ''.
	Console locate: 10@6; print: '' O''.
	Console locate: 30@5; print: ''  ''.
	Console locate: 30@6; print: '' O''.

	Console plot: 20@10.
	Console plot: 20@11.

	Console plot: 14@16.
	Console plot: 25@16.

	Console locate: 15@17; print: ''          ''.
	
	Console locate: 1@20.
	Console resetAllAttributes.
]'
! !

!CoralScriptExamples class methodsFor: 'script examples' stamp: 'AdrienBarreau 5/10/2011 18:11'!
scriptHello

  ^'[
	Console print: ''Coral is working'', String lf.
	Console print: ''Yahoo!!''
]'! !

!CoralScriptExamples class methodsFor: 'script examples' stamp: 'AdrienBarreau 10/13/2010 14:14'!
scriptPrintArguments
	"CoralInstaller generateScript: #scriptPrintArguments"

	^ '"This script takes * arguments and print them with 3 arguments-using methods"
	
[
	Console printNl:''getScriptName: '', CoralScriptWriting getScriptName.

	Console printNl: ''getAllArgumentsWithoutMinus:''.
	CoralScriptWriting getAllArgumentsWithoutMinus do: [ :each | Console printNl: each ].

	Console printNl: ''getAllArgumentsWithoutMinusButScriptName:''.
	CoralScriptWriting getAllArgumentsWithoutMinusButScriptName do: [ :each | Console printNl: each ]
]	
'! !

!CoralScriptExamples class methodsFor: 'script examples' stamp: 'DamienPollet 4/5/2012 17:19'!
scriptSynchronizeSecondFolderWithFirst
	"CoralInstaller generateScript: #scriptSynchronizeSecondFolderWithFirst"

	^ '"This script takes two paths as parameters. They should be two folders. This script will synchronize the
second folder with the first.
NB: Can be improve, this example just add the files which does not exist in the folder 2"

[ Object < #TempCoralClass ]

TempCoralClass class>>synchronizeFolder1: folder1 folder2: folder2
[
	| entries1 baseNames2 filesToCreate |
	
	entries1 := folder1 children.
	baseNames2 := folder2 children collect: [ :each | each basename ].
	filesToCreate := entries1 reject: [ :each |  baseNames2 includes: each basename ].
	
	filesToCreate do: [ :each | 
		each isFile 
			ifTrue: [ 
				| readStream |
				readStream := each readStream.
				(folder2 / (each basename)) writeStream nextPutAll: (readStream contents); close.
				readStream close
			]
			ifFalse: [ (folder2 / (each basename)) createDirectory. self synchronizeFolder1: each folder2: (folder2 / (each basename)) ]
	].
]

[
	| args fs folder1 folder2 |
	
	args := CoralScriptWriting getAllArguments.
	(args size = 3) ifFalse: [ Console<<(''Use: '', args first, '' path1 path2''). ^ self ].
	
	fs := FileSystem disk.
	folder1 := fs referenceTo: (args at:2).
	folder2 := fs referenceTo: (args at:3).
	
	(folder1 exists & folder2 exists) ifFalse: [ Console printError: ''Error: The two folders should exist.''. ^ self ].
	
	TempCoralClass synchronizeFolder1: folder1 folder2: folder2.
	Console<< (args first, '' done.'').
]	
'! !

!CoralScriptExamples class methodsFor: 'testing scripts' stamp: 'AdrienBarreau 10/13/2010 16:39'!
scriptTestCoralScriptWriting
	"Don't generate this script, but use scriptForScriptTestCoralScriptWriting"

	^ '[ TestCase < #TempCoralClass ]

TempCoralClass>>testGetAllArguments
[
	| args |
	
	args := CoralScriptWriting getAllArguments.
	
	self assert: (args size = 4).
	self assert: ((args at: #*) = #(''scriptTestCoralScriptWriting.cst'' arg1 arg2)).
	self assert: ((args at: #minus1) = #(arg3)).
	self assert: ((args at: #minus2) = #()).
	self assert: ((args at: #minus3) = #(arg4 arg5)).
]

TempCoralClass>>testGetAllArgumentsWithoutMinus
[
	| args |
	
	args := CoralScriptWriting getAllArgumentsWithoutMinus.
	
	self assert: (args size = 3).
	self assert: ((args at:1) = ''scriptTestCoralScriptWriting.cst'').
	self assert: ((args at:2) = ''arg1'').
	self assert: ((args at:3) = ''arg2'').
	
	self assert: (args = (CoralScriptWriting getAllArguments at: #*)).
]

TempCoralClass>>testGetAllArgumentsWithoutMinusButScriptName
[
	| args |
	
	args := CoralScriptWriting getAllArgumentsWithoutMinusButScriptName.
	
	self assert: (args size = 2).
	self assert: ((args at:1) = ''arg1'').
	self assert: ((args at:2) = ''arg2'').
	
	self assert: (args = (CoralScriptWriting getAllArguments at: #*) allButFirst).
	self assert: (args = CoralScriptWriting getAllArgumentsWithoutMinus allButFirst).
]

TempCoralClass>>testGetScriptName
[
	| arg |
	
	arg := CoralScriptWriting getScriptName.
	
	self assert: (arg = ''scriptTestCoralScriptWriting.cst'').
	self assert: (arg = (CoralScriptWriting getAllArguments at: #*) first).
	self assert: (arg = CoralScriptWriting getAllArgumentsWithoutMinus first).
]

[
	| flag |
	
	flag := true.
	TempCoralClass selectors do: [ :each |
		[ TempCoralClass debug: each ] on: TestFailure do: [ Console printNl: ''Test failure: '', each. flag := false ]
	].

	flag ifTrue: [ Console printNl: ''All tests were successfully passed!!''  ]
]'
! !
